"use strict";
/*
 *  Copyright (c) 2014, Facebook, Inc.
 *  All rights reserved.
 *
 *  This source code is licensed under the BSD-style license found in the
 *  LICENSE file in the root directory of this source tree. An additional grant
 *  of patent rights can be found in the PATENTS file in the same directory.
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importDefault(require("react"));
var server_1 = __importDefault(require("react-dom/server"));
var js_beautify_1 = __importDefault(require("js-beautify"));
var lodash_escaperegexp_1 = __importDefault(require("lodash.escaperegexp"));
var beautifyHTML = js_beautify_1.default.html;
var DEFAULT_ENGINE_OPTIONS = {
    doctype: '<!DOCTYPE html>',
    beautify: false,
    transformViews: true,
    babel: {
        presets: [
            '@babel/preset-react',
            [
                '@babel/preset-env',
                {
                    targets: {
                        node: 'current',
                    },
                },
            ],
            // typescript
            [
                '@babel/preset-typescript',
                {
                    isTSX: true,
                    allExtensions: true
                }
            ],
        ],
        plugins: [],
    },
};
var DEFAULT_OPTIONS = {
    settings: {
        views: [],
        env: ''
    }
};
function _createEngine(engineOptions) {
    var registered = false;
    var moduleDetectRegEx;
    var finalEngineOptions = __assign(__assign({}, DEFAULT_ENGINE_OPTIONS), (engineOptions || {}));
    function renderFile(filename, options, cb) {
        var finalOptions = __assign(__assign({}, DEFAULT_OPTIONS), (options || {}));
        // Defer babel registration until the first request so we can grab the view path.
        if (!moduleDetectRegEx) {
            // Path could contain regexp characters so escape it first.
            // options.settings.views could be a single string or an array
            moduleDetectRegEx = new RegExp([]
                .concat(typeof finalOptions.settings.views === 'string' ?
                [finalOptions.settings.views] : finalOptions.settings.views)
                .map(function (viewPath) { return '^' + lodash_escaperegexp_1.default(viewPath); })
                .join('|'));
        }
        if (finalEngineOptions.transformViews && !registered) {
            // Passing a RegExp to Babel results in an issue on Windows so we'll just
            // pass the view path.
            require('@babel/register')(__assign({ only: []
                    .concat(typeof finalOptions.settings.views === 'string' ?
                    [finalOptions.settings.views] : finalOptions.settings.views) }, finalEngineOptions.babel));
            registered = true;
        }
        var markup;
        try {
            markup = finalEngineOptions.doctype;
            var component = require(filename);
            // Transpiled ES6 may export components as { default: Component }
            component = component.default || component;
            markup += server_1.default.renderToStaticMarkup(react_1.default.createElement(component, options));
        }
        catch (e) {
            return cb ? cb(e) : e;
        }
        finally {
            if (options && options.settings.env === 'development') {
                // Remove all files from the module cache that are in the view folder.
                Object.keys(require.cache).forEach(function (module) {
                    if (moduleDetectRegEx.test(require.cache[module].filename)) {
                        delete require.cache[module];
                    }
                });
            }
        }
        if (finalEngineOptions.beautify) {
            // NOTE: This will screw up some things where whitespace is important, and be
            // subtly different than prod.
            markup = beautifyHTML(markup);
        }
        cb && cb(null, markup);
    }
    return renderFile;
}
exports.createEngine = _createEngine;
